/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : VRProblem.h                                              */
/*                                                                       */
/*       Date : May  5, 1999                                             */
/* Updated at : May  5, 1999                                             */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#ifndef _VRPROBLEM_H_
#define _VRPROBLEM_H_

class VRProblem;
typedef VRProblem* VRProblemPtr;

#include "VRPcommons.h"

#include "Customer.h"
#include "CustomerList.h"

#include "Depot.h"
#include "DepotList.h"
#include "EdgeGraph.h"
#include "TestDefinition.h"

class VRProblem
{  
   public :
     VRProblem();
     ~VRProblem();

     // copy constructor
     VRProblem(const VRProblem&);

     // returns the number of customers.
     unsigned getNumberOfCustomers() const;

     // returns the list of depots.
     DepotListPtr getDepotList() const;

     // return 0: if indx is invalid.
     getDepot(unsigned indx, Customer& a) const;

     // return 0: if indx is invalid.
     isDepot(customerId indx) const;

     // add Depot
     void addDepot(customerId& a) ;

     // returns the list of customers.
     CustomerListPtr getCustomerList() const;

     // return 0: if indx is invalid.
     getCustomer(unsigned indx, Customer& a) const;
     // return NULL: if indx is invalid.
     //CustomerPtr getCustomer(unsigned indx) const { return myCustomers->getNthElement(indx);};

     // return 0: if indx is invalid.
     getNodeWithNodeId(customerId indx, Customer& a) const;

     // 
     getNodeIndex(customerId indx) const;

     // add Customer
     void addCustomer(Customer& a) ;

     // returns the capacity of a vehicle
     vrpcapacity getMaximumCapacity() const;

     // returns the total demand 
     vrpcapacity getTotalOfDemans() const;

     // assignment operator
     VRProblem& operator=(const VRProblem&);

     operator==(const VRProblem&);

     friend ostream& operator<<(ostream&,const VRProblem&);

     inline void setTestDefinition(TestDefinition _test) { test=_test;}
     inline EdgeGraph*  getEdgeGraph() { return myEdgeGraph;};
     inline void        setEdgeGraph(EdgeGraphPtr eg) { myEdgeGraph=eg;};
     inline vrpcapacity getVehicleCapacity() { return test.getVehicleCapacity();};
   private:

    CustomerList  *myCustomers;

       DepotList  *myDepots;

    TestDefinition  test;

       EdgeGraph  *myEdgeGraph;

	vrpcapacity  maxVehicleCapacity;

	void dispose();
        void copyFrom(const VRProblem&);

}; // end of class VRProblem

#endif
